<?php

//File read & File write //

function bny_readfile($file){
	require_once(ABSPATH . 'wp-admin/includes/file.php');
	$buf = '';
	if (is_file($file) === true){
		if(WP_Filesystem()){
			global $wp_filesystem;
			$buf = $wp_filesystem->get_contents($file);
		}
	}
	return $buf;
}

function bny_writefile($file, $text){
	require_once(ABSPATH . 'wp-admin/includes/file.php');
	if(WP_Filesystem()){
		global $wp_filesystem;
		$wp_filesystem->put_contents($file, $text);
	}
}



//Execute CSS minimization script when logging out //

function mincss($c){
	$c = str_replace(array("\r\n", "\r", "\n", "\t", '  ', '   ', '    '), '', $c);
	$c = preg_replace('!/\*[^*]*\*+([^/][^*]*\*+)*/!', '', $c);
	$c = str_replace(': ', ':', $c);
	return $c;
}

function exec_css_minimization() {
	$maincss = get_template_directory() . '/style.min.css';
	$ampcss = get_template_directory() . '/amp/style.php';
	$buf = "";
	ob_start();
	get_template_part("style");
	$buf = ob_get_clean();
	$buf = mincss($buf);
	bny_writefile($maincss, $buf);
	bny_writefile($ampcss, $buf);
}



// Output sidebar file //

function bny_read_sidebar($idx) {
	$buf = "";
	ob_start();
	dynamic_sidebar($idx);
	$buf = ob_get_clean();
	return $buf;
}


function bny_static_sidebar($idx) {
	$id = 'sidebar-' . $idx;
	if (!is_active_sidebar($id)){
		return;
	}
	$outfile = get_template_directory() . '/sidebar-static.php';
	$buf  = '<aside id="bny-secondary" class="widget-area">';
	$buf .= bny_read_sidebar($idx);
	$buf .= '</aside><!-- #secondary -->';
	bny_writefile($outfile, $buf);
}




//------------------sidebar for amp----------------------------------------------

function bny_get_opt($search) {
	global $wpdb;
	$q = "SELECT option_value FROM $wpdb->options WHERE option_name = %s;";
	$r = $wpdb->get_var($wpdb->prepare($q, $search));
	return $r;
}

function bny_get_widget_num() {
	$ret = false;
	$s = maybe_unserialize(bny_get_opt('sidebars_widgets')); 
	if (isset($s)) {
		if(isset($s['sidebar-2'])) {
			$vals = $s['sidebar-2'];
		}
		if (isset($vals)) {
			foreach($vals as $key => $val) {
				if(strpos($val,'recent-posts-') !== false){
					$ret = str_replace('recent-posts-', '', $val);
				}
			}
		}
	}
	return $ret;
}

function bny_get_recentposts_num($default) {
	$ret = $default;
	if (bny_get_widget_num() !== false){
		$wnum = bny_get_widget_num();
		$s = maybe_unserialize(bny_get_opt('widget_recent-posts'));
		if (isset($s)) {
			if (isset($s[$wnum])) {
				$ret = $s[$wnum]['number'];
			}
		}
	}
	return $ret;
}


// Replacing Recent Posts Widget
function bny_replace_recent_posts($content) {
	//Number of Posts to Show  - Recent Posts Widget -

	$rnum = bny_get_recentposts_num(12);    //12 is default;

	$rest_uri = get_site_url() . '/wp-json/wp/v2/posts?per_page=' . $rnum;
	$listheight = $rnum * 60;  //Please adjust the height of one item in the list.

	$amplist = '<amp-list width="auto" height="' . $listheight . '" layout="fixed-height" src="' . $rest_uri . '" items=".">
	<template type="amp-mustache">
		<div>
			<a href="{{link}}">{{title.rendered}}</a>
		</div>
	</template>
</amp-list>';

	$content = str_replace('<amp-list></amp-list>', $amplist, $content);
	return $content;
}



// amp comverter
function bny_amp_tag_conv($content) {

	$buf  = '<html><head><meta http-equiv="Content-Type" content="text/html; charset=UTF-8" /></head><body>';
	$buf .= $content;
	$dom = new DOMDocument();
	@$dom->loadHTML($buf, LIBXML_HTML_NODEFDTD | LIBXML_NOERROR);

	//delete recent entry for REST API
	if (get_theme_mod('ck_amp_list', false)) {
		foreach ($dom->getElementsByTagName('section') as $node) {
			$nodes[] = $node;
		}
		if (isset($nodes)){
			foreach ($nodes as $node) {
				if (strpos($node->getAttribute('class'), 'widget_recent_entries') !== false){
					$newEle = $dom->createElement('amp-list' , '');
					$node->appendChild($newEle);
					foreach ($node->getElementsByTagName('ul') as $ulnode) {
						$ulnode->parentNode->removeChild($ulnode);
					}
				}
			}
		}
		unset($nodes);
	}

	//delete attributes
	$arrattr = array('style', 'target', 'onclick', 'onmousedown', 'onmouseup', 'onmouseover', 'onmouseout', 'onkeypress', 'onkeydown', 'onkeyup', 'onchange','onload', 'onunload', 'onerror', 'onabort', 'onfocus', 'onselect', 'onunload', 'onselect', 'onblur', 'onsubmit', 'onreset');
	foreach ($dom->getElementsByTagName('*') as $node) {
		$nodes[] = $node;
	}
	if (isset($nodes)){
		foreach ($nodes as $node) {
			foreach ($arrattr as $attr) {
				$node->removeAttribute($attr);
			}
		}
	}
	unset($nodes);

	//convert tags  <img>, <iframe>
	$arrtags = array('img', 'iframe', 'video', 'audio');
	foreach ($arrtags as $convtag) {
		foreach ($dom->getElementsByTagName($convtag) as $node) {
			$nodes[] = $node;
		}
		if (isset($nodes)){
			foreach ($nodes as $node) {
				$newtag = 'amp-' . $convtag;
				$newEle = $dom->createElement($newtag , '');
				$node->parentNode->appendChild($newEle);
				foreach ($node->attributes as $nodeattr) {
					$newEle->setAttribute($nodeattr->nodeName, $nodeattr->nodeValue);
				}
				$node->parentNode->removeChild($node);
			}
		}
		unset($nodes);
	}

	//remove some tags.
	$convtags = array('script', 'style', 'base', 'frame', 'frameset', 'object', 'param', 'applet', 'embed');
	foreach ($convtags as $convtag) {
		foreach ($dom->getElementsByTagName($convtag) as $node) {
			$nodes[] = $node;
		}
		if (isset($nodes)){
			foreach ($nodes as $node) {
				$node->parentNode->removeChild($node);
			}
		}
		unset($nodes);
	}

	//adding target to form
	foreach ($dom->getElementsByTagName('form') as $node) {
		$nodes[] = $node;
	}
	if (isset($nodes)){
		foreach ($nodes as $node) {
			$node->setAttribute('target', '_top');
		}
	}
	unset($nodes);

	//output
	$buf = preg_replace('/^\<html\>.*?\<body\>/', '', $dom->saveHTML());
	$buf = preg_replace('/\<\/body\>.*?\<\/html\>\n/', '', $buf);
	if (get_theme_mod('ck_amp_list', false)) {
		$buf = bny_replace_recent_posts($buf);
	}
	return $buf;
}


//amp static sidebar
function bny_amp_static_sidebar($idx) {
	$id = 'sidebar-' . $idx;
	if (!is_active_sidebar($id)){
		return;
	}
	$outfile = get_template_directory() . '/amp/sidebar-amp.php';
	$buf  = '<aside id="bny-secondary" class="widget-area">';
	$buf .= bny_amp_tag_conv(bny_read_sidebar($idx));
	$buf .= '</aside><!-- #secondary -->';
	bny_writefile($outfile, $buf);
}
